#! /usr/bin/env python

#
# see also http://docs.openstack.org/cli/quick-start/content/nova-cli-reference.html
#

import iso8601
import sys
sys.path.insert(0, '../..')
 
from datetime import datetime
import pprint
pp = pprint.PrettyPrinter(indent=4)
import json
import os

from sh import fgrep
import novaclient
from sh import nova

# from cm_credential import credentials
from cloudmesh.openstack.cm_table import table as cm_table
from cloudmesh.cm_config import cm_config

from novaclient.v1_1 import client
from novaclient.v1_1 import security_groups
from novaclient.v1_1 import security_group_rules

def donotchange(fn):
    return fn

class BaseCloud:

    """
    flavors = {}         # global var
    images = {}          # global var
    servers = {}         # global var
    credential = None    # global var
    label = None         # global var
    """

    def _clear(self):
        self.flavors = {}         # global var
        self.images = {}          # global var
        self.servers = {}         # global var
        self.credential = None    # global var
        self.label = None         # global var
        self.type = None

    def info(self):
        print "Label:", self.label
        print "Type:", self.type
        print "Flavors:", len(self.flavors)
        print "Servers:", len(self.servers)
        print "Images:", len(self.images)
 
    def connect(self):
        assert False, "Not implemented"
    def config (self, dict):
        assert False, "Not implemented"
    def find_user_id(self):
        assert False, "Not implemented"

    def get(self,type="server"):
        selection = type.lower()[0]
        list_function = self._get_servers_dict
        d = {}
        if selection == 'i':
            d = self.images
        elif selection == 'f':
            d = self.flavors
        elif selection == 's':
            d = self.servers
        elif type != None:
            print "refresh type not supported"
            assert False
        return d
    
    def _get_image_dict(self):
        print "BOOOH"
        assert False, "Not implemented"

    def _update_image_dict(self,information):
        print "BOOOH"
        assert False, "Not implemented"

    def _get_flavors_dict(self):
        print "BOOOH"
        assert False, "Not implemented"

    def _update_flavors_dict(self,information):        
        print "BOOOH"
        assert False, "Not implemented"

    def _get_servers_dict(self):
        print "BOOOH"
        assert False, "Not implemented"

    def _update_servers_dict(self,information):
        print "BOOOH"
        assert False, "Not implemented"

    def vm_create(self, name, flavor_name, image_id):
        assert False, "Not implemented"
    def vm_delete(self, id):
        assert False, "Not implemented"
    def vms_project(self, refresh=False):
        assert False, "Not implemented"
    def rename(self, old, new, id=None):
        assert False, "Not implemented"
    def usage(self, start, end, format='dict'):
        assert False, "Not implemented"
    def limits(self):        
        assert False, "Not implemented"
    def status(self, vm_id):
        assert False, "Not implemented"

    ######################################################################
    # print
    ######################################################################

    def __str__(self):
        """
        print everything but the credentials that is known about this
        cloud in json format.
        """
        information = {
            'label': self.label,
            'flavors': self.flavors,
            'servers': self.servers,
            'images': self.images}
        return json.dumps(information, indent=4)

    ######################################################################
    # get methods
    ######################################################################

    def type():
        return self.type

    def vms(self):
        return self.servers

    def status(self, vm_id):
        return self.servers[vm_id]['status']


    ######################################################################
    # set credentials
    ######################################################################

    def credentials(self, cred):
        self.credential = cred

    ######################################################################
    # set credentials
    ######################################################################

    def refresh(self, type=None):
        time_stamp = datetime.now().strftime('%Y-%m-%dT%H-%M-%SZ')
        selection = ""
        if type:
            selection = type.lower()[0]

        list_function = self._get_servers_dict
        update_function = self._update_servers_dict
        d = self.servers
        if selection == 'a' or type == None:
            self.refresh("images")
            self.refresh("flavors")
            self.refresh("servers")
            return
        elif selection == 'i':
            list_function = self._get_images_dict
            update_function = self._update_images_dict
            d = self.images
        elif selection == 'f':
            list_function = self._get_flavors_dict
            update_function = self._update_flavors_dict
            d = self.flavors
        elif selection == 's':
            list_function = self._get_servers_dict
            update_function = self._update_servers_dict
            d = self.servers
        elif type != None:
            print "refresh type not supported"
            assert False

        list = list_function()

        if len(list)  == 0:
           if selection == 'i':
               self.images = {}
           elif selection == 'f':
               self.flavors = {}
           elif selection == 's':
               self.servers = {}

        else:
            
            for information in list:
                (id, element) = update_function(information)
                d[id] = element
                d[id]['cm_refresh'] = time_stamp

    
class openstack(BaseCloud):

    type = "openstack"   # global var
    flavors = {}         # global var
    images = {}          # global var
    servers = {}         # global var
    credential = None    # global var
    label = None         # global var

    # cm_type = "openstack"
    # name = "undefined"
    
    cloud = None         # internal var for the cloud client in openstack
    user_id = None       # internal var

    _nova = nova


    ######################################################################
    # initialize
    ######################################################################
    # possibly make connext seperate
    def __init__(self, label,
                 authurl=None,
                 project=None,
                 username=None,
                 password=None,
                 cacert=None):
        """
        initializes the openstack cloud from a defould novaRC file
        locates at ~/.futuregrid.org/openstack. However if the
        parameters are provided it will instead use them
        """
        self.clear()
        self.label = label
        self.config(label)
        self.connect()
        
        
    def clear(self):
        """
        clears the data of this openstack instance, a new connection
        including reading the credentials and a refresh needs to be
        called to obtain again data.
        """
        #Todo: we may just use the name of the class instead as the type
        self._clear()
        self.type = "openstack"
        
    def connect(self):
        """
        establishes a connection to the OpenStack cloud,
        e.g. initializes the needed components to conduct subsequent
        queries.
        """
        """
           def __init__(self, username, api_key, project_id, auth_url=None,
                  insecure=False, timeout=None, proxy_tenant_id=None,
                  proxy_token=None, region_name=None,
                  endpoint_type='publicURL', extensions=None,
                  service_type='compute', service_name=None,
                  volume_service_name=None, timings=False,
                  bypass_url=None, os_cache=False, no_cache=True,
                  http_log_debug=False, auth_system='keystone',
                  auth_plugin=None,
                  cacert=None):
        """       
       
        self.cloud = client.Client(
            self.credential['OS_USERNAME'],
            self.credential['OS_PASSWORD'],
            self.credential['OS_TENANT_NAME'],
            self.credential['OS_AUTH_URL'],
            cacert=self.credential['OS_CACERT']
        )

    # BIG BUG, MUST BE DICT
    #config should have dict as parameter
    def config(self, label, dict=None):
        """
        reads in the configuration file if specified, and does some
        internal configuration.
        """
        self.label = label
        if dict == None:
            config = cm_config()
            self.credential = config.get(label,expand=True)

            #print self.credential
            
            #self.user_id = self.credential['OS_USER_ID']
            # self.credential = credentials(label)
        else:
            self.credential = {}
            self.credential['OS_USERNAME'] = username
            self.credential['OS_PASSWORD'] = password
            self.credential['OS_AUTH_URL'] = authurl
            self.credential['label'] = label
            self.credential['OS_TENANT_NAME'] = project
            self.credential['OS_CACERT'] = cacert
        """
        self._nova = nova.bake("--os-username",    self.credential.username,
                               "--os-password",    self.credential.password,
                               "--os-auth-url",    self.credential.url,
                               "--os-tenant-name", self.credential.project)
        """

    ######################################################################
    # TESTS
    ######################################################################

    def intro(self, what):
        """ used to find some methods form novaclient"""
        import inspect

        print 70 * "="
        print "class ", what.__class__.__name__, ":"
        list = inspect.getmembers(what, predicate=inspect.ismethod)
        for element in list:
            print "    ", element[0]
        try:
            pp.pprint(what.__dict__)
        except:
            return

    def novaclient_dump(self):

        # playing around to discover methods
        self.intro(self.cloud)
        self.intro(self.cloud.services)
        self.intro(self.cloud.servers)
        self.intro(self.cloud.client)

        self.intro(self.cloud.agents)
        self.intro(self.cloud.aggregates)
        self.intro(self.cloud.availability_zones)
        self.intro(self.cloud.certs)
        self.intro(self.cloud.client)
        self.intro(self.cloud.cloudpipe)
        self.intro(self.cloud.coverage)
        self.intro(self.cloud.dns_domains)
        self.intro(self.cloud.dns_entries)
        self.intro(self.cloud.fixed_ips)
        self.intro(self.cloud.flavor_access)
        self.intro(self.cloud.flavors)
        self.intro(self.cloud.floating_ip_pools)
        self.intro(self.cloud.floating_ips)
        self.intro(self.cloud.floating_ips_bulk)
        self.intro(self.cloud.fping)
        self.intro(self.cloud.hosts)
        self.intro(self.cloud.hypervisors)
        self.intro(self.cloud.images)
        self.intro(self.cloud.keypairs)
        self.intro(self.cloud.limits)
        self.intro(self.cloud.networks)
        self.intro(self.cloud.os_cache)
        self.intro(self.cloud.project_id)
        self.intro(self.cloud.quota_classes)
        self.intro(self.cloud.quotas)
        self.intro(self.cloud.security_group_rules)
        self.intro(self.cloud.security_groups)
        self.intro(self.cloud.servers)
        self.intro(self.cloud.services)
        self.intro(self.cloud.usage)
        self.intro(self.cloud.virtual_interfaces)
        self.intro(self.cloud.volume_snapshots)
        self.intro(self.cloud.volume_types)
        self.intro(self.cloud.volumes)

        now = datetime.now()

    ######################################################################
    # FIND USER ID
    ######################################################################

    def find_user_id(self):
        """
        this method returns the user id and stores it for later use.
        """
        # As i do not know how to do this properly, we just create a
        # VM and than get the userid from there

        #self.cloud.ensure_service_catalog_present(self.cloud)
        #catalog = self.cloud.client.service_catalog.catalog
        #pp.pprint(catalog['access']['user'], "User Credentials")
        #pp.pprint(catalog['access']['token'], "Token")

        #print(result)

        #sys.exit()

        try:
            self.user_id = self.credential['OS_USER_ID'] 
        except:
            if self.user_id == None:
                sample_flavor = self.cloud.flavors.find(name="m1.tiny")
                
                #hack needs to be fixed
                if self.label in "grizzly-openstack" :
                    sample_image = self.cloud.images.find(
                    id="e503bcb4-28c8-4f9f-8303-d99b9bffd568") 
                else :
                    sample_image = self.cloud.images.find(
                    id="6d2bca76-8fff-4d57-9f29-50378539b4fa")
                
                sample_vm = self.cloud.servers.create(
                    "%s-id" % self.credential["OS_USERNAME"],
                    flavor=sample_flavor,
                    image=sample_image)
                self.credential['OS_USER_ID'] = self.user_id = sample_vm.user_id
                sample_vm.delete()
        return self.user_id

    ######################################################################
    # refresh
    ######################################################################

    def _get_images_dict(self):
        return self.cloud.images.list(detailed=True)

    def _update_images_dict(self,information):
        image = information.__dict__
        id =  image['id']
        # clean not neaded info
        del image['manager']
        del image['_info']
        del image['_loaded']
        # del information.links
        return (id, image)

    def _get_flavors_dict(self):
        return self.cloud.flavors.list()

    def _update_flavors_dict(self,information):
        flavor = information.__dict__
        # clean not neaded info
        del flavor['manager']
        del flavor['_info']
        del flavor['_loaded']
        # del information.links
        id = information.name
        return (id, flavor)

    def _get_servers_dict(self):
        return self.cloud.servers.list(detailed=True)


    def _update_servers_dict(self,information):
        vm = information.__dict__
        #pp.pprint (vm)
        #pp.pprint(vm)
        delay = vm['id']
        del vm['manager']
        del vm['_info']
        del vm['_loaded']
        # del information.links
        id = vm['id']
        return (id, vm)
    
    
    ######################################################################
    # security Groups of VMS
    ######################################################################
    
    # GVL: review
    # how does this look for azure and euca? Should there be a general framework for this in the BaseCloud class
    # based on that analysis?
    #
    # comments of wht these things do and how they work are missing
    #
    def createSecurityGroup(self,default_security_group,description="no-description"):
        """
        comment is missing
        """
        protocol=""
        ipaddress=""
        max_port=""
        min_port=""
        default_security_group_id= self.cloud.security_groups.create(default_security_group,description)
        default_security_group_id=default_security_group_id.id
        
        config_security = cm_config()
        yamlFile=config_security.get();
        ruleNames=yamlFile['security']['security_groups'][default_security_group]
        for ruleName in ruleNames:
            rules=yamlFile['security']['rules'][ruleName]
            for key,value in rules.iteritems():
                if 'protocol' in key:
                    protocol=value
                elif 'max_port' in key:
                    max_port=value
                elif 'min_port' in key:
                    min_port=value
                else:
                    ip_address=value
                    
            self.cloud.security_group_rules.create(default_security_group_id, protocol, min_port,
               max_port, ip_address)
        return default_security_group


    # GVL: review
    # how does this look for azure and euca? Should there be a general framework for this in the BaseCloud class
    # based on that analysis?
    #
    # comments of wht these things do and how they work are missing
    def checkSecurityGroups(self):
        """
        TODO: comment is missing
        """
        config_security = cm_config()
        names={}
        
        securityGroups= self.cloud.security_groups.list()
        
        for securityGroup in securityGroups:
            
             names[securityGroup.name]=securityGroup.id
        
        yamlFile=config_security.get();
        if yamlFile.has_key('security'):
            default_security_group=yamlFile['security']['default']
        else :
            return None
        #default_security_group_id=names[default_security_group]       
        
        if default_security_group in names:
           return default_security_group
            
        else:
           return self.createSecurityGroup(default_security_group)

    # GVL: review
    # how does this look for azure and euca? Should there be a general framework for this in the BaseCloud class
    # based on that analysis?
    #
    # comments of wht these things do and how they work are missing
    #
    def get_public_ip(self):
        """
        TODO: comment is missing
        """
        return self.cloud.floating_ips.create()
    
    # GVL: review
    # how does this look for azure and euca? Should there be a general framework for this in the BaseCloud class
    # based on that analysis?
    #
    # comments of wht these things do and how they work are missing
    #
    def assign_public_ip(self,serverid,ip):
        """
        comment is missing
        """
        self.cloud.servers.add_floating_ip(serverid,ip)
        
    ######################################################################
    # create a vm
    ######################################################################
    def vm_create(self, name, flavor_name, image_id,security_groups = None,key_name = None):
        """
        create a vm
        """
        
        if not key_name == None :
            if not self.check_key_pairs(key_name):
                config = cm_config()
                dict_t = config.get()
                key = dict_t['keys']['keylist'][key_name]
                if not 'ssh-rsa' in key:
                    key = open(key,"r").read()
                self.upload_key_pair(key,key_name)
                
        
        print image_id
        vm_flavor = self.cloud.flavors.find(name=flavor_name)
        vm_image = self.cloud.images.find(id=image_id)
        
        if key_name == None :
            vm = self.cloud.servers.create(name,
                                           flavor=vm_flavor,
                                           image=vm_image,security_groups=security_groups
                                           )
        else :
            vm = self.cloud.servers.create(name,
                                           flavor=vm_flavor,
                                           image=vm_image,
                                           key_name = key_name,security_groups=security_groups
                                           )
        delay = vm.user_id  # trick to hopefully get all fields
        data = vm.__dict__
        del data['manager']
        # data['cm_name'] = name
        # data['cm_flavor'] = flavor_name
        # data['cm_image'] = image_id
        return {str(data['id']):  data}

    ######################################################################
    # delete vm(s)
    ######################################################################

    def vm_delete(self, id):
        """
        delete a single vm and returns the id
        """

        vm = self.cloud.servers.delete(id)
        # return just the id or None if its deleted
        return vm

    @donotchange
    def vms_delete(self, ids):
        """
        delete many vms by id. ids is an array
        """
        for id in ids:
            print "Deleting %s" % self.servers[id]['name']
            vm = self.vm_delete(id)

        return ids

    ######################################################################
    # list user images
    ######################################################################

    @donotchange
    def vms_user(self, refresh=False):
        """
        find my vms
        """
        user_id = self.find_user_id()

        time_stamp = datetime.now().strftime('%Y-%m-%dT%H-%M-%SZ')
        if refresh:
            self.refresh("servers")

        result = {}
        
        for (key, vm) in self.servers.items():
            if vm['user_id'] == self.user_id:
                result[key] = vm

        return result

    ######################################################################
    # list project vms
    ######################################################################

    def vms_project(self, refresh=False):
        """
        find my vms
        """
        user_id = self.find_user_id()

        time_stamp = datetime.now().strftime('%Y-%m-%dT%H-%M-%SZ')
        if refresh:
            self.refresh("images")

        result = {}
        for (key, vm) in self.servers.items():
            result[key] = vm

        return result

    ######################################################################
    # delete images from a user
    ######################################################################

    @donotchange
    def vms_delete_user(self):
        """
        find my vms and delete them
        """

        user_id = self.find_user_id()
        vms = self.find('user_id', user_id)
        self.vms_delete(vms)

        return

    ######################################################################
    # find
    ######################################################################

    @donotchange
    def find(self, key, value=None):
        ids = []
        if key == 'user_id' and value == None:
            value = self.user_id
        for (id, vm) in self.servers.items():
            if vm[str(key)] == value:
                ids.append(str(vm['id']))

        return ids

    ######################################################################
    # rename
    ######################################################################

    def rename(self, old, new, id=None):
        all = self.find('name', old)
        print all
        if len(all) > 0:
            id = all[0]
            vm = self.cloud.servers.update(id, new)
        return

    ##### TODO: BUG WHY ARE TGERE TWO REINDEX FUNCTIONS?
    
    @donotchange
    def reindex(self, prefixold, prefix, index_format):
        all = self.find('user_id')
        counter = 1
        for id in all:
            old = self.servers[id]['name']
            new = prefix + index_format % counter
            print "Rename %s -> %s, %s" % (old, new, self.servers[id]['key_name'])
            if old != new:
                vm = self.cloud.servers.update(id, new)
            counter += 1

    @donotchange
    def reindex(self, prefix, index_format):
        all = self.find('user_id')
        counter = 1
        for id in all:
            old = self.servers[id]['name']
            new = prefix + index_format % counter
            print "Rename %s -> %s, %s" % (old, new, self.servers[id]['key_name'])
            # if old != new:
            #    vm = self.cloud.servers.update(id, new)
            counter += 1

    ######################################################################
    # TODO
    ######################################################################
    """
    refresh just a specific VM
    delete all images that follow a regualr expression in name
    look into sort of images, flavors, vms
    """

    ######################################################################
    # EXTRA
    ######################################################################
    # will be moved into one class

    @donotchange
    def table_col_to_dict(self, body):
        result = {}
        for element in body:
            key = element[0]
            value = element[1]
            result[key] = value
        return result

    @donotchange
    def table_matrix(self, text, format=None):
        lines = text.splitlines()
        headline = lines[0].split("|")
        headline = headline[1:-1]
        for i in range(0, len(headline)):
            headline[i] = str(headline[i]).strip()

        lines = lines[1:]

        body = []

        for l in lines:
            line = l.split("|")
            line = line[1:-1]
            entry = {}
            for i in range(0, len(line)):
                line[i] = str(line[i]).strip()
                if format == "dict":
                    key = headline[i]
                    entry[key] = line[i]
            if format == "dict":
                body.append(entry)
            else:
                body.append(line)
        if format == 'dict':
            return body
        else:
            return (headline, body)

    ######################################################################
    # CLI call of ussage
    ######################################################################

    # will be moved into utils
    @donotchange
    def parse_isotime(self, timestr):
        """Parse time from ISO 8601 format"""
        try:
            return iso8601.parse_date(timestr)
        except iso8601.ParseError as e:
            raise ValueError(e.message)
        except TypeError as e:
            raise ValueError(e.message)

    def usage(self, start, end, format='dict'):
        """ returns the usage information of the tennant"""

        # print 70 * "-"
        # print self.cloud.certs.__dict__.get()
        # print 70 * "-"

        #tenantid = "member"  # not sure how to get that
        iso_start = self.parse_isotime(start)
        iso_end = self.parse_isotime(end)
        #print ">>>>>", iso_start, iso_end
        #info = self.cloud.usage.get(tenantid, iso_start, iso_end)

        # print info.__dict__
        #sys.exit()

        (start,rest) = start.split("T") # ignore time for now
        (end,rest) = end.split("T") # ignore time for now
        result = fgrep(self._nova("usage", "--start", start, "--end", end), "|")
        
        (headline, matrix) = self.table_matrix(result)
        headline.append("Start")
        headline.append("End")
        matrix[0].append(start)
        matrix[0].append(end)

        if format == 'dict':
            result = {}
            for i in range(0, len(headline)):
                result[headline[i]] = matrix[0][i]
            return result
        else:
            return (headline, matrix[0])

    ######################################################################
    # CLI call of absolute-limits
    ######################################################################
    def limits(self):
        """ returns the usage information of the tennant"""

        list = []

        info = self.cloud.limits.get()
        del info.manager
        rates = info.__dict__['_info']['rate']

        for rate in rates:
            limit_set = rate['limit']
            print limit_set
            for limit in limit_set:
                list.append(limit)

        return list
    
    def check_key_pairs(self,key_name):
        
        allKeys = self.cloud.keypairs.list()
        for key in allKeys:
            if key.name in key_name:
                return True
        return False
                
        
    
    ######################################################################
    # Upload Key Pair
    ######################################################################
    def upload_key_pair(self,publickey,name):
        """ Uploads key pair """
        
        try:
            self.cloud.keypairs.create(name,publickey)
        except Exception, e:
            return 1, e
            
        return (0 ,'Key added successfully')

    ######################################################################
    # Delete Key Pair
    ######################################################################
    def delete_key(self,name):
        """ delets key pair """
        
        try:
            self.cloud.keypairs.delete(name)
        except Exception, e:
            return (1, e)
            
        return (0 , 'Key deleted successfully')
    
    ######################################################################
    # List Security Group
    ######################################################################
    def sec_grp_list(self):
        """ lists all security groups """
        try:
            return self.cloud.security_groups.list()
        except Exception, e:
            print e

##########################################################################
# MAIN FOR TESTING
##########################################################################

if __name__ == "__main__":

    """
    cloud = openstack("india-openstack")
    
    name ="%s-%04d" % (cloud.credential["OS_USERNAME"], 1)
    out = cloud.vm_create(name, "m1.tiny", "6d2bca76-8fff-4d57-9f29-50378539b4fa")
    pp.pprint(out)  
    
    """

    cloud = openstack("grizzly-openstack")
    keys = cloud.list_key_pairs()
    for key in keys:
        print key.name
    """
    print cloud.find_user_id()

    """

    """
    for i in range (1,3):
        name ="%s-%04d" % (cloud.credential["OS_USERNAME"], i)
        out = cloud.vm_create(name, "m1.tiny", "6d2bca76-8fff-4d57-9f29-50378539b4fa")
        pp.pprint(out)
    """

    """
    print cloud.find('name', name)
    """

    # cloud.rename("gvonlasz-0001","gregor")




""" run with

nosetests -v --nocapture

or

nosetests -v

"""
import time
import sys
from cloudmesh.openstack.cm_table import table as cm_table
from cloudmesh.cm_config import cm_config
from cloudmesh.openstack.cm_compute import openstack
import json
import pprint
pp = pprint.PrettyPrinter(indent=4)

def HEADING(txt):
    print
    print "#", 70 * '#'
    print "#", txt
    print "#", 70 * '#'

class Test_openstack:

    cloud_label = "grizzly-openstack"

    def setup(self):
        self.cloud = openstack(self.cloud_label)

    def tearDown(self):
        pass
    def test_00_check_label(self):
        HEADING("00 INFO OPENSTACK LABEL")
        print self.cloud_label
        assert self.cloud.label == self.cloud_label

    def test_01_limit(self):
        HEADING("01 INFO OPENSTACK LIMIT")
        print >> sys.stderr, json.dumps(self.cloud.limits(), indent=4)

    def test_02_images(self):
        HEADING("02 INFO OPENSTACK IMAGES")
        self.cloud.refresh('images')
        print json.dumps(self.cloud.images, indent=4)
        # pp.pprint(self.cloud.images)
        # doing a simple test as tiny is usually 512
        #assert self.cloud.flavors['m1.tiny']['ram'] == 512
        print "Currently running vms:", len(self.cloud.images)
        #we assume cloud is always busy which may actually not true
        # we shoudl start our own vm and than probe for it for now > 0 will do
        assert self.cloud.images > 0

    def test_03_flavor(self):
        HEADING("03 INFO OPENSTACK FLAVORS")
        self.cloud.refresh('flavors')
        print json.dumps(self.cloud.flavors, indent=4)

        # doing a simple test as tiny is usually 512
        assert self.cloud.flavors['m1.tiny']['ram'] == 512

    def test_04_start_vm(self):
        HEADING("04 START VM")
        result = self.cloud.vm_create("gregor-test-001","m1.tiny","e503bcb4-28c8-4f9f-8303-d99b9bffd568")
        print result
        assert len(result.keys()) > 0
        
    def test_04_vms(self):
        HEADING("04 INFO OPENSTACK VMS")
        self.cloud.refresh('servers')
        print json.dumps(self.cloud.servers, indent=4)
        # we assume that there are always images running
        assert len(self.cloud.servers) > 0

    def test_05_refresh(self):
        HEADING("05 INFO OPENSTACK REFRESH")
        self.cloud.refresh()
        pp.pprint(self.cloud)

        assert self.cloud.images > 0

    """
    def test_0??_usage(self):
        result = self.cloud.usage("2000-01-01T00:00:00", "2013-12-31T00:00:00")
        print json.dumps(result, indent=4)
        assert ['Instances'] > 0 
    """
        
    def test_06_table(self):
        HEADING("06 INFO OPENSTACK TABLES")
        self.test_03_flavor()
        table = cm_table()
        columns = ["id", "name", "ram", "vcpus"]

        table.create(self.cloud.flavors, columns, header=True)
        print table

        table = cm_table()
        columns = ["id", "name", "ram", "vcpus"]

        table.create(self.cloud.flavors, columns, format='HTML', header=True)
        print table

        table = cm_table()
        columns = ["id", "name", "ram", "vcpus"]

        table.create(self.cloud.flavors, columns, format='%12s', header=True)
        print table

        assert table != None
    """
    def test_07_start_delete_vm(self):        
        name ="%s-%04d" % (self.cloud.credential["OS_USERNAME"], 1)
        out = self.cloud.vm_create(name, "m1.tiny", "6d2bca76-8fff-4d57-9f29-50378539b4fa")

        pp.pprint(out)
        print json.dumps(out, indent=4)

        key = out.keys()[0]
        id = out[key]["id"]
        print id

        vm = self.cloud.vm_delete(id)
        print vm
    """

    def test_08_user_vms(self):
        HEADING("08 INFO LIST VMS FROM USER")
        list = self.cloud.vms_user(refresh=True)
        print json.dumps(list, indent=4)

    def test_09_delete_all_user_vms(self):
        HEADING("09 INFO OPENSTACK DELETE VMS FROM USER")

        
        self.cloud.refresh()

        user_id = self.cloud.find_user_id()
        vm_ids = self.cloud.find('user_id', user_id)

        servers = self.cloud.servers
        print servers

        list = self.cloud.vms_delete_user()

        self.cloud = openstack(self.cloud_label)
        self.cloud.refresh()
        self.cloud.info()
        
        vm_ids = self.cloud.find('user_id', user_id)
        self.cloud.info()
        
        time.sleep(2)
        self.cloud.refresh()

        while len(vm_ids) > 0:
            vm_ids = self.cloud.find('user_id', user_id)
            self.cloud.refresh("servers")
            self.cloud.info()
            time.sleep(1)
        
        print "vms",  vm_ids
        assert vm_ids == []

    def test_10_info(self):
        HEADING("10 INFO OPENSTACK TEST")
        self.cloud.refresh()
        time.sleep(3)
        self.cloud.info()
        self.cloud.info()





import os
import yaml
import string
from random import choice

class cloudmesh_cloud:
    """Provides credentials for various cloud platforms""" 
    CLOUD_DEFNS = 'cloudmesh_clouds.yaml'

    def __init__(self, profiledata, defaultproj, projectlist, cloudname):
        self._profile = profiledata
        self._projectlist = projectlist
        self._defaultproj = defaultproj
        self._cloudname = cloudname
        self._data = yaml.safe_load(open(self.CLOUD_DEFNS, "r"))[cloudname]

    @property
    def username(self):
        return self._profile['username']

    @property
    def email(self):
        return self._profile['e-mail']

    @property
    def cloudname(self):
        return self._cloudname

    @property
    def projects(self):
        return self._projectlist

    @property
    def defaultproject(self):
        return self._defaultproj

    @property
    def clouddefaults(self):
        return self._data['default']

    @property
    def credentials(self):
        return self._data['credentials']

    @property
    def data(self):
        d = dict(self._data)
        del d['cm_admin']
        return d

    @property
    def admin_data(self):
        return self._data

    def newpass(self):
        """Credit: http://stackoverflow.com/questions/3854692/generate-password-in-python"""
        chars = string.letters + string.digits
        length = 12
        return ''.join([choice(chars) for _ in range(length)])

    def initialize_cloud_user(self):
        raise Exception("Not implemented")



import pickle
from sh import fgrep
from sh import nova
from sh import tail
from datetime import datetime
import json
import sys
import os
import pprint
pp = pprint.PrettyPrinter(indent=4)

# import shelve
from cloudmesh.cm_config import cm_config
#from openstack.cm_compute import openstack as os_client

#Error Cannot Import Openstack
from cloudmesh.openstack.cm_compute import openstack

from cloudmesh.eucalyptus.eucalyptus_new import eucalyptus
from cloudmesh.azure.cm_azure import cm_azure as azure 

try:
    # from sh import fgmetric
    from fgmetric.FGMetricsAPI import FGMetricsAPI
    # OR
    # from sh import fgmetric
except:
    # TODO THIS IS NOT HOW WE DO IT, SETUP CONTAINS ALL REQUEMENTS
    # FIX SETUP
    print "---------------------"
    print "fgmetric not imported"
    print "---------------------"
    pass
try:
    from bson import json_util
except:
    # TODO THIS IS NOT HOW WE DO IT, SETUP CONTAINS ALL REQUEMENTS
    # FIX SETUP
    print "--------------------------------"
    print "Please run 'pip install pymongo'"
    print "--------------------------------"

    #

# WHY ARE WE NOT MORE CLEANLY SEPARATING METRIC FROM THIS?
class cloudmesh:

    ######################################################################
    # global variables that define the information managed by this class
    ######################################################################

    datastore = "data/clouds.txt"

    # dict that holds vms, flavors, images for al iaas
    clouds = {}

    # array with keys from the user
    keys = []

    ######################################################################
    # variables that we can most likely eliminate
    ######################################################################

    # user needs to come from credential ...
    user = "gvonlasz"

    ######################################################################
    # initialization methods
    ######################################################################

    def __init__(self):
        self.clear()
        #Read Yaml File to find all the cloud configurations present
        self.config();
        try:
            self.metric_api = FGMetricsAPI()
        except:
            pass

    def clear(self):
        self.clouds = {}
        self.keys = []
        self.user = "gvonlasz"

    ######################################################################
    # some metric methods
    ######################################################################

    def get_metrics_cli(self, args):
        """ Get usage data from FG Metric CLI"""
        """ This is replica with get_metrics but using CLI instead of API """
        """
            Args:
                args (dict): parameters for CLI with option
            Return:
                (dict): output of fgmetric in a dict type
            Raise:
                n/a
        """
        try:
            res = fgmetric(
                args)  # args should be list-lized before send it out as a parameter
            return json.loads(res, object_hook=json_util.object_hook)
        except:
            pass

    def get_metrics(self, args):
        """Get usage data from FG Metrics"""

        if not self.metric_api:
            return

        try:
            args["user"] = args["user"] or self.user
            self._set_metric_api_vars(args)
            # print args
            stats = self.metric_api._set_dict_vars()
            metrics = self.metric_api.get_stats()
            stats["stats"] = metrics
            self.metrics = stats
            # print self.metrics
        except:
            print sys.exc_info()
            pass
        return self.metrics

    def get_realtime_metrics(self, args):
        """ get real-time usage data from FG Metrics"""

        # IN DEVELOPMENT
        if not self.metric_api:
            return

        try:
            print 1#
        except:
            pass

    def _set_metric_api_vars(self, args):
        self.metric_api.set_date(args["s_date"], args["e_date"])
        self.metric_api.set_metric(
            "count runtime cores mem disks")  # args["metric"])
        self.metric_api.set_user(args["user"])
        self.metric_api.set_cloud(args["cloud"])
        self.metric_api.set_hostname(args["host"])
        self.metric_api.set_period(args["period"])

    ######################################################################
    # the configuration method that must be called to get the cloud info
    ######################################################################

    def config(self):
        """
        reads the cloudmesh yaml file that defines which clouds build
        the cloudmesh
        """

        configuration = cm_config()
        pp.pprint (configuration)

        for cloud_name in configuration.keys():
            print "--------------------"

            try:
                credential = configuration.get(key=cloud_name)
                cloud_type = credential['cm_type']

                print credential

                print ">>>>>>>", cloud_name, cloud_type

                if cloud_type in ['openstack','eucalyptus']:
                    print "AAAAA"
                    self.clouds[cloud_name] = {'cm_type': cloud_type, 'credential': credential}
                    print "BBBB"
                    try:
                        self.update(cloud_name, cloud_type)
                        self.clouds[cloud_name] = {'cm_type': cloud_type, 'credential': credential}
                    except:
                        print "ERROR: can not connect to", cloud_name
                    print "CCCC"

                    
            except Exception, e:
              print "ERROR: Not a cloud:", cloud_name , e
        return

    ######################################################################
    # importnat get methods
    ######################################################################

    def get(self):
        """returns the dict that contains all the information"""
        return self.clouds

    ######################################################################
    # important print methods
    ######################################################################
    # includes sanitizing to remove the credentials
    ######################################################################

    def __str__(self):
        tmp = self._sanitize()
        print tmp

    def _sanitize(self):
        # copy the self.cloud
        # delete the attributes called credential for all clouds

        all_keys =  self.clouds.keys()
        for cloud in all_keys:
            self.clouds[cloud]['credential'] = {}

        return self.clouds

    def dump(self):
        tmp = self._sanitize()
        print json.dumps(tmp, indent=4)

    ######################################################################
    # the refresh method that gets upto date information for cloudmesh
    # If cloudname is provided only that cloud will be refreshed 
    # else all the clouds will be refreshed
    ######################################################################

    def refresh(self, cloud_name=None):
        print "Refershing cloud %s" % cloud_name
        servers = {}        
        cloud =None;
        
        if(cloud_name == None):
            all_clouds = self.clouds.keys()
            for cloud_name in all_clouds :
                try:
                    type = self.clouds[cloud_name]['cm_type']

                    if type == 'openstack':
                        cloud = openstack(cloud_name)
                        
                    elif type == 'eucalyptus':
                        # Where do i find the project name ? Is there a defaul one ? 
                        cloud = eucalyptus(cloud_name, 'fg-82')
                        
                    elif type == 'azure':
                        cloud = azure.cm_azure()
                       
                    cloud.refresh()
                    self.clouds[cloud_name]['flavors'] = cloud.flavors
                    self.clouds[cloud_name]['images'] = cloud.images
                    self.clouds[cloud_name]['servers'] = cloud.servers
                
                except Exception, e:
                    print e
        else:
            try:
                type = self.clouds[cloud_name]['cm_type']

                if type == 'openstack':
                    cloud = openstack(cloud_name)
                    
                elif type == 'eucalyptus':
                    # Where do i find the project name ? Is there a defaul one ? 
		    # this is obvious;ly wrong as the tennent comes from profile
		    # TODO EUCALYPTUS DOES NOT YET WORK

                    cloud = eucalyptus(cloud_name, 'fg-82')
                    
                elif type == 'azure':
                    cloud = azure.cm_azure()
                   
                cloud.refresh()
                self.clouds[cloud_name]['flavors'] = cloud.flavors
                self.clouds[cloud_name]['images'] = cloud.images
                self.clouds[cloud_name]['servers'] = cloud.servers
                
            except Exception, e:
                    print e
        return cloud

    def update(self, name, type):
        servers = self.refresh(name)
        self.clouds[name].update({'name': name,
                                  'cm_type': type,
                                  "servers": servers})
        return

    def add(self, name, type):
        try:
            self.clouds[name]
            print "Error: Cloud %s already exists" % name
        except:
            self.update(name, type)

    """
    def get_keys(self):
        return self.keys

    def refresh_keys(self):
        self.keys = []
        result = fgrep(tail(nova("keypair-list"), "-n", "+4"),"-v","+")
        for line in result:
            (front, name, signature, back) = line.split("|")
            self.keys.append(name.strip())
        return self.keys


    def refresh(self):
        keys = self.refresh_keys()
        for cloud in keys:
            self.refresh(cloud)

        # p = Pool(4)
        # update = self.refresh
        # output = p.map(update, keys)

    """

    ######################################################################
    # saves and reads the dict to and from a file
    ######################################################################
    def save(self):
        tmp = self._sanitize()
        file = open(self.datastore, 'wb')
        # pickle.dump(self.keys, file)
        pickle.dump(tmp, file)
        file.close()

    def load(self):
        file = open(self.datastore, 'rb')
        # self.keys = pickle.load(file)
        self.clouds = pickle.load(file)
        ''' above returns:
        [u'gvonlasz']
         So, call pickle again to get more:
            {'india': {'name': 'india',
            'servers': {u'2731c421-d985-44ce-91bf-2a89ce4ba033': {'cloud': 'india',
            'id': u'2731c421-d985-44ce-91bf-2a89ce4ba033',
            'ip': u'vlan102=10.1.2.85, 149.165.158.7',
            'name': u'gvonlasz-001',
            'refresh': '2013-02-11 20:30:04.472583',
            'status': u'ACTIVE'},
            ...
        '''
        self.clouds = pickle.load(file)
        file.close()

    ######################################################################
    # TODO: convenient +, += functions to add dicts with cm_type
    ######################################################################

    def __add__(self, other):
        """
        type based add function c = cloudmesh(...); b = c + other
        other can be a dict that contains information about the object
        and it will be nicely inserted into the overall cloudmesh dict
        the type will be identified via a cm_type attribute in the
        dict Nn attribute cm_cloud identifies in which cloud the
        element is stored.
        """
        if other.cm_type == "image":
            print "TODO: not implemented yet"
            return
        elif other.cm_type == "vm":
            print "TODO: not implemented yet"
            return
        elif other.cm_type == "flavor":
            print "TODO: not implemented yet"
            return
        elif other.cm_type == "cloudmesh":
            print "TODO: not implemented yet"
            return
        else:
            print "Error: %s type does not exist", cm_type
            print "Error: Ignoring add"
            return

    def __iadd__(self, other):
        """
        type based add function c = cloudmesh(...); c += other other
        can be a dict that contains information about the object and
        it will be nicely inserted into the overall cloudmesh dict the
        type will be identified via a cm_type attribute in the dict.
        Nn attribute cm_cloud identifies in which cloud the element is
        stored.
        """
        if other.cm_type == "image":
            print "TODO: not implemented yet"
            return
        elif other.cm_type == "vm":
            print "TODO: not implemented yet"
            return
        elif other.cm_type == "flavor":
            print "TODO: not implemented yet"
            return
        elif other.cm_type == "cloudmesh":
            print "TODO: not implemented yet"
            return
        else:
            print "Error: %s type does not exist", cm_type
            print "Error: Ignoring add"
            return

##########################################################################
# MAIN METHOD FOR TESTING
##########################################################################

if __name__ == "__main__":

    c = cloudmesh()
    print c.clouds
    """
    c.config()

    c.dump()


    c = cloud_mesh()

    c.refresh()
    c.add('india', 'openstack')
    c.add('sierra', 'openstack')
    c.refresh_keys()
    c.dump()
    c.save()
    print 70 * "-"
    c.clear()
    c.dump()
    print 70 * "-"
    c.load()
    c.dump()
    print 70 * "-"
    """

    """
    india_os = {
        "OS_TENANT_NAME" : '',
        "OS_USERNAME" : '',
        "OS_PASSWORD" : '',
        "OS_AUTH_URL" : '',
        }


    (attribute, passwd) = fgrep("OS_PASSWORD","%s/.futuregrid/openstack/novarc" % os.environ['HOME']).replace("\n","").split("=")

    india_os['OS_PASSWORD'] = passwd



    username = india_os['OS_USERNAME']
    password = india_os['OS_PASSWORD']
    authurl = india_os['OS_AUTH_URL']
    tenant = india_os['OS_TENANT_NAME']

    print password
    '''
    username = os.environ['OS_USERNAME']
    password = os.environ['OS_PASSWORD']
    authurl = os.environ['OS_AUTH_URL']
    '''
    india = cloud_openstack("india", authurl, tenant, username, password)

    india._vm_show("gvonlasz-001")
    india.dump()
    india._vm_show("gvonlasz-001")
    india.dump()
    """



from fabric.api import *
import ConfigParser
import pickle
import os
import commands
from sh import azure as _azure
import getpass
import pprint
import re


from multiprocessing import Pool as _Pool
global vmName, vmImage, vmPassword, maxparallel
maxparallel = 5
CONFIGPATH = 'config.cfg'
DBFILEPATH = 'database.db'

config = ConfigParser.ConfigParser()
config.read(CONFIGPATH)
regex = re.compile("\x1b\[(?:\d{1,2}m?)?", re.UNICODE)


vm_start = azure.bake("vm", "start")
vm_restart = azure.bake("vm", "restart")
vm_shutdown = azure.bake("vm", "shutdown")
vm_list = azure.bake("vm", "list")
vm_create = azure.bake("vm", "create")
vm_show = azure.bake("vm", "show")
vm_delete = azure.bake("vm", "delete")


def activate():
    settingsFilePath = config.get('azure', 'publishsettings_file_path')
    _azure('account', 'clear')
    errmsg = 'No account information found'
    cmd = 'azure account import \'' + settingsFilePath+'\''

    text = commands.getstatusoutput(cmd)
    if not errmsg in text[1]:
        print 'Activated'
    else:
        print 'There was an error while activation'


def _selectImage():
    images = _buildAzureImageDict()
    print 'Please select Image'
    allItems = images.items()
    # for (arg1, arg2) in allItems:
    for item in allItems:
        print item[0] + "\t" + item[1][0] + "\t" + item[1][1] + "\t" + item[1][2]

    while 1:
        var = raw_input("Image : ")
        if var in images:
            print images[var][0]
            return images[var][0]
        else:
            print "Incorrect Image name"
    return var


def create(index, name=config.get('azure', 'username')):
    global vmName, vmImage, vmPassword
    vmImage = config.get('azure', 'image')
    userName = config.get('azure', 'username')
    vmPassword = config.get('azure', 'password')
    vmName = name

    if(vmImage == ''):
        print('image name not set, use set:image,value)')
    elif(userName == ''):
        print('image name not set, use set:username,value)')
    elif(vmPassword == ''):
        print('image name not set, use set:password,value)')
    else:
        """Creates a number of vms with the labels prefix-0 to prefix-<number-1>. It uses a threadpool"""
        pool = _Pool(processes=maxparallel)
        list = range(0, int(index))
        result = pool.map(_boot, list)
        # _boot(index)
        print result


def start(name):
    result = vm_start(name)
    print result


def restart(name):
    result = vm_restart(name)
    print result


def shutdown(name):
    result = vm_shutdown(name)
    print result


def _boot(index):
    global vmName, vmImage, vmPassword
    number = str(index).zfill(3)
    name = '%s-%s' % (vmName, number)
    vm = {'vmname': name,
          'username': config.get('azure', 'username'),
          'image': vmImage,
          'password': vmPassword
          }

    # cmd = 'azure vm create %(vmname)s %(image)s %(username)s --ssh --location "East US" %(password)s' % vm
    # print cmd
    result = vm_create("%(vmname)s" % vm,
                       "%(image)s" % vm,
                       "%(username)s" % vm,
                       "--ssh",
                       "--location",
                       "East US",
                       "%(password)s" % vm)
    return result


def list():
        result = vm_list()
        print result


def show(name):
        result = vm_show()
        print result


def delete(name):
        result = vm_delete()
        print result


def set(option, value=''):
    cfgFile = open(CONFIGPATH, 'w')
    if(option == 'password'):
        config.set('azure', 'password', value)
    if(option == 'image'):
        value = _selectImage()
        config.set('azure', 'image', value)
    if(option == 'username'):
        config.set('azure', 'username', value)
    config.write(cfgFile)
    cfgFile.close()


def _buildAzureImageDict():
    allText = _azure('vm', 'image', 'list')
    imageNamePrefix = 'azure_img'
    imageNameCounter = 0
    allLines = allText.splitlines()
    cleanLines = []
    counter = 2
    images = {}
    # skip counter number of lines
    for line in allLines:
        if 'data' in line:
            if not counter:
                cleanLines.append(line)
            else:
                counter = counter - 1

    for line in cleanLines:
        imageNameCounter = imageNameCounter+1
        l = []
        #(name, id, os) = line.split()
        # l.append(name)
        # l.append(id)
        # l.append(os)

        l.append(line.split()[1])
        l.append(line.split()[2])
        l.append(line.split()[3])
        images[imageNamePrefix+str(imageNameCounter)] = l

    return images


def _getVmDict(name):
    text = vm_show(name)

    allLines = text.splitlines()
    cleanLines = []
    vm = {}
    # skip counter number of lines

    for line in allLines:
        if 'data' in line:
            cleanLines.append(regex.sub("", line))

    for line in cleanLines:
        line = line.replace(':', '"')
        splits = line.split('"')
        vm[str(splits[1].strip())] = str(splits[2].strip())
    allItems = vm.items()
    return vm


def save():
    text = vm_list()
    allLines = text.splitlines()
    cleanLines = []
    # skip counter number of lines
    azure = {}
    vms = {}
    counter = 2
    # allines = allines [2:-1]
    for line in allLines:
        if 'data' in line:
            # not needed make sure you have in all lines only the lines that need to be parsed
            # see above [2:-1] you may have to use [2:-2] dependent if new line
            # is added, find out what the right thing is ...

            if not counter:
                cleanLines.append(regex.sub("", line))
            else:
                counter = counter - 1

    for line in cleanLines:
        splits = line.split()
        # (arg1, arg2) = line.split();
        # vms[arg1] = _getVMDict(arg2)
        vms[splits[1]] = _getVmDict(splits[2])

    # _getVmDict('shwetaVM1-000');
    azure['name'] = 'azure'
    azure['servers'] = vms
    pickle.dump(azure, open(DBFILEPATH, "wb"))
    azure = pickle.load(open(DBFILEPATH, "rb"))
    print azure



"""Cloud Mesh: managing multiple virtual machines in Clouds

This project allows you to manage multiple virtual machines.
In future you will be able ta add multipl eclouds.
"""

from setuptools import setup, find_packages
import sys
import os

version = "0.8.1"


######################################################################
# REQUIREMENTS
######################################################################

requires=[
    'setuptools',
    'pip',
    'docopt',
    'pyyaml',
    'Flask',
    'Flask-FlatPages',
    'Flask-WTF',
    'paramiko',
    'blessings',
    'fabric',
    'progress',
    'sh',
    "console",
    "pymongo",
    "sphinxcontrib-blockdiag",
    "python-novaclient",
    "python-keystoneclient",
    "python-ldap",
    "apache-libcloud"
],

install_requires = []

for package in requires:
    try:
        import package
    except ImportError:
        install_requires.append(package)

def fpath(name):
    return os.path.join(os.path.dirname(__file__), name)


def read(fname):
    return open(fpath(fname)).read()


def desc():
    info = read('README.md')
    try:
        return info + '\n\n' + read('CHANGES.txt')
    except IOError:
        return info



######################################################################
# CLASSIFIER
######################################################################

classifiers = """\
Intended Audience :: Developers
Intended Audience :: Education
Intended Audience :: Science/Research
Development Status :: 4 - Beta
Intended Audience :: Developers
License :: OSI Approved :: Apache Software License
Programming Language :: Python
Topic :: Database
Topic :: Software Development :: Libraries :: Python Modules
Operating System :: POSIX :: Linux
Programming Language :: Python :: 2.7
Operating System :: MacOS :: MacOS X
Topic :: Scientific/Engineering
Topic :: System :: Clustering
Topic :: System :: Distributed Computing
"""

if sys.version_info < (2, 7):
    _setup = setup

    def setup(**kwargs):
        if "classifiers" in kwargs:
            del kwargs["classifiers"]
        _setup(**kwargs)


######################################################################
# SETUP
######################################################################

doclines = __doc__.split("\n")

setup(
    name='flask_cm',
    version=version,
    description=doclines[0],
    classifiers=filter(None, classifiers.split("\n")),
    long_description=desc(),
    keywords='Cloud FutureGrid Flask farmework',
    maintainer='Gregor von Laszewski',
    maintainer_email="laszewski@gmail.com",
    author='Gregor von Laszewski',
    author_email='laszewski@gmail.com',
    url='https://github.com/futuregrid/flask_cm',
    license='Apache 2.0',
    package_dir={'': '.'},
    packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),

    # include_package_data=True,
    zip_safe=False,

    entry_points={
        'console_scripts': [
                'fg-manage = cloudmesh.cm_rc:main',
#                'fg-csh = fgvirtualcluster.FGShell:main',
             ]},

    install_requires=install_requires


#    scripts=['bin/cm', 'bin/cm']

)

#pip install -e git+https://github.com/openstack/python-novaclient.git#egg=python-novaclient

# pycrypto



import sys
sys.path.insert(0, '..') 
from datetime import datetime
import pprint 
pp = pprint.PrettyPrinter(indent=4)
import json
import os

from libcloud.compute.types import Provider
from libcloud.compute.providers import get_driver
from libcloud.compute.types import NodeState

import libcloud.security
import time
from sh import fgrep


#from cm_table import table as cm_table
from openstack.cm_credential import credentials as credentials_rc
class eucalyptus:

    type = "eucalyptus"
    sizes = {}
    images = {}
    nodes = {}
    credentials = None
    accesskey = None
    secretkey = None
    project = "fg82"



    #
    # change to gregors credential class
    #

    def activate_project(self, project):

        self.credentials = credentials_rc("eucalyptus")
        self.credentials.location = ".futuregrid/india/eucalyptus/" + project + "/eucarc"

        self.credentials.type('eucalyptus')

        self.access_key = self.credentials._get_rc_variable ("accesskey")
        self.secret_key = self.credentials._get_rc_variable ("secretkey")

        print self.access_key
        print self.secret_key


    # url = 


    # don't forget to source your novarc file

    cloud = None
    label = None
    #user_id = None
    
   
    def vms(self):
        return self.servers

    """
    def credentials(self, cred):
        self.credential = cred
    """

    def __init__(self, label,project=None):
        """
        initializes the openstack cloud from a defould novaRC file
        locates at ~/.futuregrid.org/openstack. However if the
        parameters are provided it will instead use them
        """
        self.config(label,project)
        self.connect()

    def config(self,label,project=None):
        """
        reads in the configuration file if specified, and does some
        internal configuration.
        """
        self.clear()

        self.label= label 
        if project == None:
            self.activate_project("fg82")
        else:
            self.activate_project(project)


    def clear(self):
        """
        clears the data of this openstack instance, a new connection
        including reading the credentials and a refresh needs to be
        called to obtain again data.
        """
        type = "eucalyptus"
        self.sizes = {}
        self.images = {}
        self.nodes = {}
        #self.credential = None
        self.cloud = None
        self.user_id = None
    
    def connect(self):
        """
        establishes a connection to the eucalyptus cloud,
        e.g. initializes the needed components to conduct subsequent
        queries.
        """


        Driver = get_driver(Provider.EUCALYPTUS)
        
        path = os.environ['HOME'] + "/" + self.credentials.location.replace("/eucarc", "")

        os.environ['CA_CERTS_PATH'] = path

        self.cloud = Driver(self.accesskey, 
                            self.secretkey,
                            secure=False, 
                            host="149.165.146.135", # this is not quite right but will do for now 
                            port=8773, # this is not quite right but will do for now 
                            path="/services/Eucalyotus") # this is not quite right but will do for now 
    

        
    def __str__ (self):
        """
        print everything but the credentials that is known about this
        cloud in json format.
        """
        information = {
            'label': self.label,
            'flavors': self.sizes,
            'vms': self.nodes,
            'images' : self.images}
        return json.dumps(information, indent=4)

    def type():
        return self.type

    def refresh(self, type=None):

        time_stamp = datetime.now().strftime('%Y-%m-%dT%H-%M-%SZ')
        selection = ""
        if type:
            selection = type.lower()[0]
            all = selection == 'a'
        else:
            all = True
        
        if selection == 'i' or all:

            list = self.cloud.list_images()

            for information in list:
                image=information.__dict__
                del information._uuid
                del information.driver
                self.images[information.id] = image
                self.images[information.id]['cm_refresh'] = time_stamp
            

                
if __name__ == "__main__":


  credential_test = False
  flavor_test = False
  table_test = False
  image_test = True
  vm_test = False
  cloud_test = False

  """
  if credential_test:
      credential = cm_config('india-openstack')
      print credential
  """


  cloud = eucalyptus("india-eucalyptus")


  """
  if flavor_test or table_test:
      cloud.refresh('flavors')
      print json.dumps(cloud.flavors, indent=4)

  if table_test:
      table = cm_table()
      columns = ["id", "name", "ram", "vcpus"]

      table.create(cloud.flavors, columns, header=True)
      print table

      table.create(cloud.flavors, columns, format='HTML', header=True)
      print table

      table.create(cloud.flavors, columns, format='%12s', header=True)11
      print table
  """
  if image_test:
      cloud.refresh('images')
      print "hey shweta"
      print json.dumps(cloud.images, indent=4)
#-      pp.pprint (cloud.images)
  """
  if vm_test:
      cloud.refresh('vms')
       
      print json.dumps(cloud.images, indent=4)
      
  if cloud_test:
      cloud.refresh()
      print cloud


  print cloud.find_user_id()
  """
                                                    



class Client(object):
    """ Provides a mock class for testing.  Represents the keystone client. """
    mockusername = 'foo'
    mocktenants = ['foo']

    class mock_object(object):
        def __init__(self, **kwds):
            self.__dict__.update(kwds)

    class mock_users(object):
        def __init__(self, users):
            self.users = users
        def create(self, name, password, email):
            pass
        def update_password(self, user, password):
            pass
        def list(self):
            return self.users

    class mock_roles(object):
        def __init__(self, roles):
            self.roles = roles
        def add_user_role(self, user, role, tenant):
            pass
        def list(self):
            return self.roles

    class mock_tenants(object):
        def __init__(self, tenants):
            self.tenants = tenants
        def list(self):
            return self.tenants
        def create(self, tname):
            pass

    def __init__(self, username, password, tenant_name, auth_url, cacert):
        self.user = Client.mock_object(name=self.mockusername)
        self.role = Client.mock_object(name='_member_')
        self.tenantlist = []
        for t in self.mocktenants:
            self.tenantlist.append(Client.mock_object(name=t))

    @property
    def users(self):
        return self.mock_users([self.user])

    @property
    def roles(self):
        return self.mock_roles([self.role])

    @property
    def tenants(self):
        return self.mock_tenants(self.tenantlist)

if __name__ == "__main__":
    muser = Client('foo', 'foo', 'foo', 'foo', 'foo')
    print muser.users.list()[0].name
    print muser.roles.list()[0].name
    print muser.tenants.list()[0].name



from cm_config import cm_config
import os
from string import Template
import base64
import hashlib
import sys

def key_fingerprint(key_string):
    key = base64.decodestring(key_string)
    fp_plain = hashlib.md5(key).hexdigest()
    return ':'.join(a+b for a,b in zip(fp_plain[::2], fp_plain[1::2]))

def key_validate(type,filename):
    if type.lower() == "file":
        try :
            keystring = open(filename, "r").read()
        except :
            return False
    else :
        keystring = file
    
    try :
        type, key_string, comment = keystring.split()
        data = base64.decodestring(key_string)
        int_len = 4
        str_len = struct.unpack('>I', data[:int_len])[0] # this should return 7

        if data[int_len:int_len+str_len] == type:
            return True
    except Exception, e:
        print e
        return False

class cm_keys:

    filename = None
    
    def __init__(self, filename=None):
        """initializes based on cm_config and returns pointer to the keys dict."""
        # Check if the file exists
        if filename == None:
            self.config = cm_config()
        else:
            self.filename = self._path_expand(filename)
            try:
                with open(self.filename): pass
            except IOError:
                print 'ERROR: cm_keys, file "%s" does not exist' % self.filename
                sys.exit()
            self.config = cm_config(self.filename)

    def type(self, name):
        try:
            value = self._getvalue(name)
            if value.startswith("ssh"):
                return "string"
            else:
                return "file"
        except:
            return "keys"
        
    def _getvalue (self,name):
        if name == 'keys':
            return self.config.data["cloudmesh"]["keys"]
        elif name == 'default':
            key = self.config.data["cloudmesh"]["keys"]["default"]
        else:
            key = name
        value = self.config.data["cloudmesh"]["keys"]["keylist"][key]
        return value

    def __getitem__(self,name):
        value = self._getvalue(name)
        key_type = self.type(name)
        
        if key_type == "file":
            value = self._get_key_from_file(value)

        return value
            
    def __setitem__(self, name, value):
        if name == 'default':
            self.config.data["cloudmesh"]["keys"]["default"] = value
            return
        else:
            self.config.data["cloudmesh"]["keys"]["keylist"][name] = value

    def delete(self, name):
        """ not tested"""
        if name == 'default':
            key = self.config.data["cloudmesh"]["keys"]["default"]
            newdefault
        else:
            key = name
            
        del self.config.data["cloudmesh"]["keys"]["keylist"][key]

        if newdefault:
            if len(self.config.data["cloudmesh"]["keys"]["keylist"]) > 0:
                default = self.config.data["cloudmesh"]["keys"]["keylist"][0]
        else:
            default = None
            
    def _path_expand(self,text):
        """ returns a string with expanded variavble """
        template = Template(text)
        result = template.substitute(os.environ)
        result = os.path.expanduser(result)
        return result

    def _get_key_from_file(self, filename):
        return open(self._path_expand(os.path.expanduser(filename)), "r").read()
                
    def setdefault(self, name):
        """sets the default key"""
        self.config.data["cloudmesh"]["keys"]["default"] = name

    def default(self):
        """sets the default key"""
        return self.config.userkeys('default')
        
    def names(self):
        """returns all key names in an array"""
        return self.config.data["cloudmesh"]["keys"]["keylist"].keys()

    def validate(self, line):
        """validates if a default key os ok and follows 'keyencryptiontype keystring keyname'"""

    def __str__(self):
        """returns the dict in a string representing the project"""
        return str(self.config)

    def update(self):
        """writes the updated dict to the config"""
    
    def fingerprint(self, name):
        value = self.__getitem__(name)
        t, keystring, comment = value.split()
        return key_fingerprint(keystring)
